
program LZHTest;

uses
  LZH;

const
  MaxBuf = 4096;                       { Must be bigger than the biggest chunk being asked for. }

type
  BufType = array[1..MaxBuf] of byte;
  BufPtr = ^BufType;

var
  InBuf, OutBuf : BufPtr;
  infile, Outfile : file;
  s : string;
  Bytes_Written : longint;
  Size : longint;
  Temp : word;


{$F+}
  procedure GetBlock(var Target; NoBytes : word; var Actual_Bytes : word);
  const
    Posn : word = 1;
    Buf : word = 0;
  var
    Temp : word;
  begin
    if (Posn > Buf) or (Posn + NoBytes > succ(Buf)) then
      begin
        if Posn > Buf then
          begin
            blockread(infile, InBuf^, MaxBuf, Buf);
            write('+');
          end
        else
          begin
            move(InBuf^[Posn], InBuf^[1], Buf - Posn);
            blockread(infile, InBuf^[Buf - Posn], MaxBuf - (Buf - Posn), Temp);
            Buf := Buf - Posn + Temp;
            write('+');
          end;
        if Buf = 0 then
          begin
            Actual_Bytes := 0;
            writeln;
            exit;
          end;
        Posn := 1;
      end;
    move(InBuf^[Posn], Target, NoBytes);
    inc(Posn, NoBytes);
    if Posn > succ(Buf) then
      Actual_Bytes := NoBytes - (Posn - succ(Buf))
    else Actual_Bytes := NoBytes;
  end;


  procedure PutBlock(var Source; NoBytes : word; var Actual_Bytes : word);
  const
    Posn : word = 1;
  var
    Temp : word;
  begin
    if NoBytes = 0 then                { Flush condition }
      begin
        blockwrite(Outfile, OutBuf^, pred(Posn), Temp);
        exit;
      end;
    if (Posn > MaxBuf) or (Posn + NoBytes > succ(MaxBuf)) then
      begin
        blockwrite(Outfile, OutBuf^, pred(Posn), Temp);
        Posn := 1;
      end;
    move(Source, OutBuf^[Posn], NoBytes);
    inc(Posn, NoBytes);
    Actual_Bytes := NoBytes;
  end;
{$F-}

BEGIN
  if (paramcount <> 3) then
    begin
      writeln('Usage:lzhuf e(compression)|d(uncompression) infile outfile');
      halt(1);
    end;
  s := paramstr(1);
  if not(s[1] in ['D', 'E', 'd', 'e']) then
    halt(1);
  assign(infile, paramstr(2));
  reset(infile, 1);
  assign(Outfile, paramstr(3));
  rewrite(Outfile, 1);
  new(InBuf);
  new(OutBuf);
  if (upcase(s[1]) = 'E') then
    begin
      Size := filesize(infile);
      blockwrite(Outfile, Size, sizeof(longint));
      LZHPack(Bytes_Written, GetBlock, PutBlock);
      PutBlock(Size, 0, Temp);
    end
  else
    begin
      blockread(infile, Size, sizeof(longint));
      LZHUnPack(Size, GetBlock, PutBlock);
      PutBlock(Size, 0, Temp);
    end;
  dispose(OutBuf);
  dispose(InBuf);
  close(infile);
  close(Outfile);
END.

